CONTAINER = {'__contains__'}
HASHABLE = {'__hash__'}
ITERABLE = {'__iter__'}
ITERATOR = ITERABLE | {'__next__'}
REVERSABLE = ITERABLE | {'__reversed__'}
GENERATOR = ITERATOR | {'send', 'throw', 'close'}
SIZED = {'__len__'}
CALLABLE = {'__call__'}
COLLECTION = SIZED | ITERABLE | CONTAINER
SEQUENCE = REVERSABLE | COLLECTION | {'index', 'count'}
MUTABLE_SEQUENCE = SEQUENCE | {'__setitem__', '__delitem__',
                               'insert', 'append', 'reverse',
                               'extend', 'pop', 'remove',
                               '__iadd__'}
BYTE_STRING = SEQUENCE
SET = COLLECTION | {'__le__', '__lt__', '__eq__', '__ne__',
                    '__gt__', '__ge__', '__and__', '__or__',
                    '__sub__', '__xor__', 'isdisjoint'}
MUTABLE_SET = SET | {'add',  'discard', 'clear', 'pop',
                     'remove', '__ior__', '__iand__',
                     '__ixor__', '__isub__'}
MAPPING = COLLECTION | {'keys', 'items', 'values', 'get',
                        '__eq__', '__ne__'}
MUTABLE_MAPPING = MAPPING | {'__setitem__', '__delitem__',
                             'pop', 'popitem', 'clear',
                             'update', 'setdefault'}
MAPPING_VIEW = SIZED
ITEM_VIEW = MAPPING_VIEW | SET
KEYS_VIEW = MAPPING_VIEW | COLLECTION
AWAITABLE = {'__await__'}
COROUTINE = AWAITABLE | {'send', 'throw', 'close'}
ASYNC_ITERABLE = {'__aiter__'}
ASYNC_ITERATOR = ASYNC_ITERABLE | {'__anext__'}
ASYNC_GENERATOR = ASYNC_ITERATOR | {'asend', 'athrow', 'aclose'}


from typing import Any, Iterable
from hasattrs.attributes import CONTAINER, HASHABLE, ITERABLE, ITERATOR, REVERSABLE
from hasattrs.attributes import GENERATOR, SIZED, CALLABLE, COLLECTION, SEQUENCE
from hasattrs.attributes import MUTABLE_SEQUENCE, BYTE_STRING, SET, MUTABLE_SET
from hasattrs.attributes import MAPPING, MUTABLE_MAPPING, MAPPING_VIEW
from hasattrs.attributes import ITEM_VIEW, KEYS_VIEW, AWAITABLE, COROUTINE
from hasattrs.attributes import ASYNC_ITERABLE, ASYNC_ITERATOR, ASYNC_GENERATOR


def has_attrs(obj: Any, attrs: Iterable[str]) -> bool:
    return all(hasattr(obj, a) for a in attrs)


def has_container_attrs(obj: Any) -> bool:
    return has_attrs(obj, CONTAINER)


def has_hashable_attrs(obj: Any) -> bool:
    return has_attrs(obj, HASHABLE)


def has_iterable_attrs(obj: Any) -> bool:
    return has_attrs(obj, ITERABLE)


def has_iterator_attrs(obj: Any) -> bool:
    return has_attrs(obj, ITERATOR)


def has_reversible_attrs(obj: Any) -> bool:
    return has_attrs(obj, REVERSABLE)


def has_generator_attrs(obj: Any) -> bool:
    return has_attrs(obj, GENERATOR)


def has_sized_attrs(obj: Any) -> bool:
    return has_attrs(obj, SIZED)


def has_callable_attrs(obj: Any) -> bool:
    return has_attrs(obj, CALLABLE)


def has_collection_attrs(obj: Any) -> bool:
    return has_attrs(obj, COLLECTION)


def has_sequence_attrs(obj: Any) -> bool:
    return has_attrs(obj, SEQUENCE)


def has_mutable_sequence_attrs(obj: Any) -> bool:
    return has_attrs(obj, MUTABLE_SEQUENCE)


def has_byte_string_attrs(obj: Any) -> bool:
    return has_attrs(obj, BYTE_STRING)


def has_set_attrs(obj: Any) -> bool:
    return has_attrs(obj, SET)


def has_mutable_set_attrs(obj: Any) -> bool:
    return has_attrs(obj, MUTABLE_SET)


def has_mapping_attrs(obj: Any) -> bool:
    return has_attrs(obj, MAPPING)


def has_mutable_mapping_attrs(obj: Any) -> bool:
    return has_attrs(obj, MUTABLE_MAPPING)


def has_mapping_view_attrs(obj: Any) -> bool:
    return has_attrs(obj, MAPPING_VIEW)


def has_item_view_attrs(obj: Any) -> bool:
    return has_attrs(obj, ITEM_VIEW)


def has_keys_view_attrs(obj: Any) -> bool:
    return has_attrs(obj, KEYS_VIEW)


def has_awaitable_attrs(obj: Any) -> bool:
    return has_attrs(obj, AWAITABLE)


def has_coroutine_attrs(obj: Any) -> bool:
    return has_attrs(obj, COROUTINE)


def has_async_iterable_attrs(obj: Any) -> bool:
    return has_attrs(obj, ASYNC_ITERABLE)


def has_async_iterator_attrs(obj: Any) -> bool:
    return has_attrs(obj, ASYNC_ITERATOR)


def has_async_generator_attrs(obj: Any) -> bool:
    return has_attrs(obj, ASYNC_GENERATOR)